#!/usr/bin/env python 
'''
Copyright (C) 2008 Thomas Holder, http://sf.net/users/speleo3/
Distributed under the terms of the GNU General Public License v2

This program was inspired by http://www.cavediving.de/svg2th2.py
'''

# first try to assure that the inkscape extensions dir in in PYTHONPATH
try:
	import inkex
except:
	import sys, os
	programfiles = os.getenv('PROGRAMFILES')
	if programfiles:
		sys.path.append(programfiles + '\\Inkscape\\share\\extensions')
	try:
		import commands
		statusoutput = commands.getstatusoutput('inkscape -x')
		assert statusoutput[0] == 0
		sys.path.append(statusoutput[1])
	except:
		sys.path.append('/usr/share/inkscape/extensions')

import inkex, simplepath, simpletransform, simplestyle, math, re

def transformParams(mat, params):
	new = []
	for i in range(0, len(params), 2):
		new.append(mat[0][0]*params[i]+mat[0][1]*params[i+1]+mat[0][2])
		new.append(mat[1][0]*params[i]+mat[1][1]*params[i+1]+mat[1][2])
	return new

def descrim(mat):
	return math.sqrt(abs(mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0]))

class Th2Line:
	def __init__(self):
		self.options = {}
		self.points = []
	def append(self, params):
		self.points.append(" ".join(str(i) for i in params))
	def close(self):
		self.options['close'] = 'on'
	def output(self, type = 'wall'):
		line_cmd = "line " + type
		for k,v in self.options.iteritems():
			line_cmd += " -%s %s" % (k,v)
		print line_cmd
		print "  " + "\n  ".join(self.points)
		print "endline\n"

class Th2Output(inkex.Effect):
	def __init__(self):
		inkex.Effect.__init__(self)
		self.OptionParser.add_option("--scale",    type="int",     dest="scale",    default=100)
		self.OptionParser.add_option("--dpi",      type="int",     dest="dpi",      default=90)
		self.OptionParser.add_option("--linetype", type="string",  dest="linetype", default="wall")
		self.OptionParser.add_option("--layers",   type="string",  dest="layers",   default="all")
		self.OptionParser.add_option("--images",   type="inkbool", dest="images",   default=False)
		self.OptionParser.add_option("--stroke",   type="float",   dest="stroke",   default=0.0)
		self.OptionParser.add_option("--nolpe",    type="inkbool", dest="nolpe",    default=True)
		self.OptionParser.add_option("--lay2scr",  type="inkbool", dest="lay2scr",  default=True)

		inkex.NSS['therion'] = 'http://therion.speleo.sk/therion'

	def i2d_affine(self, node):
		m2 = simpletransform.parseTransform(node.get('transform'))
		while True:
			node = node.getparent()
			if not node:
				break
			t1 = node.get('transform')
			if not t1:
				continue
			m1 = simpletransform.parseTransform(t1)
			m2 = simpletransform.composeTransform(m1, m2)
		m2 = simpletransform.composeTransform(self.r2d, m2)
		return m2
	
	def print_scrap_begin(self, id, test):
		if test:
			print '\nscrap %s -scale [ %f ]\n' % (id, self.meters_per_unit)

	def print_scrap_end(self, test):
		if test:
			print "endscrap\n"

	def output(self):
		doc_width = inkex.unittouu(self.document.getroot().get('width'))
		doc_height = inkex.unittouu(self.document.getroot().get('height'))

		self.r2d = [[1, 0, 0],[0, -1, doc_height]]
		viewBox = self.document.getroot().get('viewBox')
		if viewBox:
			viewBox = [float(i) for i in viewBox.split()]
			m1 = [[doc_width / viewBox[2], 0, -viewBox[0]], [0, doc_height / viewBox[3], -viewBox[1]]]
			self.r2d = simpletransform.composeTransform(self.r2d, m1)

		self.classes = {}
		stylenode = self.xpathSingle('/svg:svg/svg:defs/svg:style')
		if stylenode != None:
			pattern = re.compile(r'\.(\w+)\s*\{(.*?)\}')
			for x in pattern.finditer(stylenode.text):
				self.classes[x.group(1)] = simplestyle.parseStyle(x.group(2))

		print '''encoding  utf-8
##XTHERION## xth_me_area_adjust 0 0 %f %f
##XTHERION## xth_me_area_zoom_to 50
''' % (doc_width, doc_height)

		if self.options.images:
			images = self.document.xpath('/svg:svg/svg:g//svg:image', namespaces=inkex.NSS)
			for node in reversed(images):
				params = [ inkex.unittouu(node.get('x')), inkex.unittouu(node.get('y')) ]
				mat = self.i2d_affine(node)
				params = transformParams(mat, params)
				href = node.get(inkex.addNS('href', 'xlink'))
				print '##XTHERION## xth_me_image_insert {%f 1 1.0} {%f {}} "%s" 0 {}' % \
					(params[0], params[1], href)

		meters_per_inch = 0.0254
		self.meters_per_unit = self.options.scale * meters_per_inch / self.options.dpi

		self.print_scrap_begin('scrap1', not self.options.lay2scr)

		if self.options.layers == 'current':
			self.output_scrap(self.current_layer)
		else:
			layers = self.document.xpath('/svg:svg/svg:g', namespaces=inkex.NSS)
			if len(layers) == 0:
				layers = [ self.document.getroot() ]
			for layer in layers:
				if self.options.layers == 'visible':
					style = simplestyle.parseStyle(layer.get('style', ''))
					if style.get('display') == 'none':
						continue
				self.output_scrap(layer)

		self.print_scrap_end(not self.options.lay2scr)

	def output_scrap(self, layer):
		id = layer.get('id')
		self.print_scrap_begin(id, self.options.lay2scr)

		points = layer.xpath('.//*[@therion:role="point"]', namespaces=inkex.NSS)
		for node in points:
			params = [ inkex.unittouu(node.get('cx')), inkex.unittouu(node.get('cy')) ]
			mat = self.i2d_affine(node)
			params = transformParams(mat, params)
			type = node.get(inkex.addNS('type', 'therion'), 'station')
			options = node.get(inkex.addNS('options', 'therion'), '')
			print "point %f %f %s %s" % (params[0], params[1], type, options)

		paths = layer.xpath('.//svg:path', namespaces=inkex.NSS)
		for node in paths:
			mat = self.i2d_affine(node)

			if self.options.stroke > 0.0:
				s = {}
				classname = node.get('class', '').split()
				for c in classname:
					s.update(self.classes.get(c, {}))
				s.update(simplestyle.parseStyle(node.get('style', '')))
				stroke = inkex.unittouu(s.get('stroke-width', '1')) * descrim(mat)
				if round(stroke, 3) != round(self.options.stroke, 3):
					continue

			d = node.get(inkex.addNS('original-d', 'inkscape'))
			if not self.options.nolpe or not d:
				d = node.get('d')
			p = simplepath.parsePath(d)

			type = node.get(inkex.addNS('type', 'therion'), self.options.linetype)

			th2line = None
			for cmd,params in p:
				if cmd == 'M':
					if th2line != None:
						th2line.output(type)
					th2line = Th2Line()
				if cmd == 'Z':
					th2line.close()
				else:
					th2line.append(transformParams(mat, params))
			if th2line != None:
				th2line.output()

		self.print_scrap_end(self.options.lay2scr)

if __name__ == '__main__':
	e = Th2Output()
	e.affect()
